"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var Monoid_1 = require("fp-ts/lib/Monoid");
var Option_1 = require("fp-ts/lib/Option");
var Semigroup_1 = require("fp-ts/lib/Semigroup");
var Ord_1 = require("fp-ts/lib/Ord");
/**
 * 'RetryPolicy' is a 'Monoid'. You can collapse multiple strategies into one using 'concat'.
 * The semantics of this combination are as follows:
 *
 * 1. If either policy returns 'None', the combined policy returns
 * 'None'. This can be used to inhibit after a number of retries,
 * for example.
 *
 * 2. If both policies return a delay, the larger delay will be used.
 * This is quite natural when combining multiple policies to achieve a
 * certain effect.
 *
 * @example
 *
 * // One can easily define an exponential backoff policy with a limited
 * // number of retries:
 * const limitedBackoff = monoidRetryPolicy.concat(exponentialBackoff(50), limitRetries(5))
 */
exports.monoidRetryPolicy = Monoid_1.getFunctionMonoid(Option_1.getApplyMonoid(__assign({}, Semigroup_1.getJoinSemigroup(Ord_1.ordNumber), { empty: 0 })))();
/** Retry immediately, but only up to `i` times. */
exports.limitRetries = function (i) {
    return function (status) { return (status.iterNumber >= i ? Option_1.none : Option_1.some(0)); };
};
/**
 * Add an upperbound to a policy such that once the given time-delay
 * amount *per try* has been reached or exceeded, the policy will stop
 * retrying and fail.
 */
exports.limitRetriesByDelay = function (maxDelay, policy) {
    return function (status) {
        return policy(status).chain(function (delay) {
            return delay >= maxDelay ? Option_1.none : Option_1.some(delay);
        });
    };
};
/** Constant delay with unlimited retries */
exports.constantDelay = function (delay) {
    return function () { return Option_1.some(delay); };
};
/**
 * Set a time-upperbound for any delays that may be directed by the
 * given policy. This function does not terminate the retrying. The policy
 * capDelay(maxDelay, exponentialBackoff(n))` will never stop retrying. It
 * will reach a state where it retries forever with a delay of `maxDelay`
 * between each one. To get termination you need to use one of the
 * 'limitRetries' function variants.
 */
exports.capDelay = function (maxDelay, policy) {
    return function (status) { return policy(status).map(function (delay) { return Math.min(maxDelay, delay); }); };
};
/**
 * Grow delay exponentially each iteration.
 * Each delay will increase by a factor of two.
 */
exports.exponentialBackoff = function (delay) {
    return function (status) { return Option_1.some(delay * Math.pow(2, status.iterNumber)); };
};
/**
 * Initial, default retry status. Exported mostly to allow user code
 * to test their handlers and retry policies.
 */
exports.defaultRetryStatus = {
    iterNumber: 0,
    cumulativeDelay: 0,
    previousDelay: Option_1.none
};
/**
 * Apply policy on status to see what the decision would be.
 */
exports.applyPolicy = function (policy, status) {
    var previousDelay = policy(status);
    return {
        iterNumber: status.iterNumber + 1,
        cumulativeDelay: status.cumulativeDelay + previousDelay.getOrElse(0),
        previousDelay: previousDelay
    };
};
//# sourceMappingURL=index.js.map